// @ts-check

/**
 * Clean compiled template files location
 */
function tplClean() {
  const del = require("del");
  const paths = require("../paths");
  return del(paths.join(paths.frontend.dest.base, "**/*"));
}

/**
 * Build only template files ( blade templates and php)
 */
function tplBuild(callback) {
  const gulp = require("gulp");
  const coreSync = require("./coreSync");
  const tplComponents = require("./tplComponents");
  const tplFragments = require("./tplFragments");
  const tplLaravel = require("./tplLaravel");
  const tplLayouts = require("./tplLayouts");
  const tplMiddlewares = require("./tplMiddlewares");
  const tplRoutes = require("./tplRoutes");
  const tplServices = require("./tplServices");
  const tplUtils = require("./tplUtils");

  return gulp.parallel([
    coreSync,
    tplComponents,
    tplFragments,
    tplLaravel,
    tplLayouts,
    tplMiddlewares,
    tplRoutes,
    tplServices,
    tplUtils,
  ])(callback);
}

/**
 * Watch only template files ( blade templates and php)
 */
function tplWatch(callback) {
  const gulp = require("gulp");
  const paths = require("../paths");
  const { getWatchPath } = require("../utils");
  const tplComponents = require("./tplComponents");
  const tplFragments = require("./tplFragments");
  // const tplLaravel = require("./tplLaravel");
  const tplLayouts = require("./tplLayouts");
  const tplMiddlewares = require("./tplMiddlewares");
  const tplRoutes = require("./tplRoutes");
  const tplServices = require("./tplServices");
  const tplUtils = require("./tplUtils");
  const { join, frontend } = paths;
  const { src } = frontend;

  gulp.watch(getWatchPath(join(src.components, "/**/*.php")), tplComponents);
  gulp.watch(getWatchPath(join(src.fragments, "/**/*.php")), tplFragments);
  gulp.watch(getWatchPath(join(src.layouts, "/**/*.php")), tplLayouts);
  gulp.watch(getWatchPath(join(src.middlewares, "/**/*.php")), tplMiddlewares);
  gulp.watch(getWatchPath(join(src.routes, "/**/*.{php,json}")), tplRoutes);
  gulp.watch(getWatchPath(join(src.services, "/**/*.php")), tplServices);
  gulp.watch(getWatchPath(join(src.utils, "/**/*.php")), tplUtils);
}

/**
 * Exposse build and watch task separetely
 */
module.exports = {
  tplClean,
  tplBuild,
  tplWatch,
};
