const webpack = require("webpack");
// const CleanWebpackPlugin = require("clean-webpack-plugin");
const CopyPlugin = require("copy-webpack-plugin");
const { isProduction } = require("../config");
const { getProjectJsGlobals } = require("../utils");
const { getOutputName, getCopySetting } = require("./helpers");
const paths = require("../paths");

let plugins = [
  new webpack.DefinePlugin(
    getProjectJsGlobals().reduce((output, def) => {
      output[def.name] = def.value;
      return output;
    }, {})
  ),
  // new webpack.ProvidePlugin({
  //   Vue: ["vue/dist/vue.esm.js", "default"],
  // }),
  // new CleanWebpackPlugin({
  //   cleanStaleWebpackAssets: true,
  // }),
  new CopyPlugin({
    patterns: [
      getCopySetting("fonts", true),
      getCopySetting("images", false),
      getCopySetting("media", false),
    ],
  }),
  // FIXME: move to favicons-webpack-plugin, it breaks at the moment
  // ].concat(require("./plugins-favicons")());
];

if (isProduction) {
  const MiniCssExtractPlugin = require("mini-css-extract-plugin");
  const PostCSSAssetsPlugin = require("postcss-assets-webpack-plugin");

  plugins = plugins.concat([
    new MiniCssExtractPlugin({
      filename: getOutputName("entry", "css"),
      chunkFilename: getOutputName("chunk", "css"),
      ignoreOrder: true,
    }),
    new PostCSSAssetsPlugin({
      test: /\.css$/,
      log: false,
      plugins: [require("postcss-sort-media-queries")()],
    }),
  ]);
} else {
  plugins.push(new webpack.HotModuleReplacementPlugin());
}

if (isProduction) {
  const { WebpackManifestPlugin } = require("webpack-manifest-plugin");
  const WorkboxPlugin = require("workbox-webpack-plugin");

  plugins = plugins.concat([
    new WebpackManifestPlugin({
      // the public url is prepended in the `assets.blade.php` automated partial
      publicPath: paths.frontend.dest.folders.assets + "/",
      // filter out compiled partials and license files
      filter: ({ name }) => {
        if (name.endsWith(".LICENSE.txt")) {
          return false;
        }
        return true;
      },
      map: (file) => {
        let { name } = file;
        // Problem: hash is contained in the manifest keys, @see open issues
        // https://github.com/danethurber/webpack-manifest-plugin/issues/210
        // https://github.com/danethurber/webpack-manifest-plugin/issues/211
        // so here we manually strip it with a regex
        const matches = name.match(/.+\/.*?\.(.+\.).+$/);
        if (matches && matches[1]) {
          name = name.replace(matches[1], "");
        }

        return {
          ...file,
          name,
        };
      },
    }),
    new webpack.BannerPlugin({
      banner: require("../utils/banner")(),
      raw: true,
      entryOnly: true,
      exclude: /^vendors/,
    }),
    // @see https://developers.google.com/web/tools/workbox/guides/codelabs/webpack
    // @see https://developers.google.com/web/tools/workbox/reference-docs/latest/module-workbox-webpack-plugin.InjectManifest#InjectManifest
    new WorkboxPlugin.GenerateSW({
      swDest: paths.join(paths.frontend.dest.assets, "service-worker.js"),
      clientsClaim: true,
      skipWaiting: true,
    }),
    // new WebpackBar(),
  ]);
}

if (process.env.DEV_ANALYZE === "true") {
  const BundleAnalyzerPlugin = require("webpack-bundle-analyzer")
    .BundleAnalyzerPlugin;
  plugins = plugins.concat([
    new BundleAnalyzerPlugin({
      reportFilename: paths.join(
        paths.frontend.dest.folders.assets,
        "stats.json"
      ),
    }),
  ]);
}

module.exports = () => plugins;
